/**
  *  \file peng.h
  *  \brief Portable Shiplist Compiler
  */
#ifndef PENG_H_INCLUDED
#define PENG_H_INCLUDED

#include <stdio.h>

#define DEFAULT_VERSION 2

#define countof(x) (sizeof(x) / sizeof(x[0]))

#define TINT -1
#define TLONG -2
#define TINTARRAY -3
#define TLONGARRAY -4

#define VERSION "0.32"

#define MAGIC_CHECKSUM 338817

typedef unsigned char uchar;

struct Assignment {
    short       type;           /* TINT, TLONG or length of string */
    short       offset;         /* offset in data record */
    const char* name;           /* name for Penguin v2, may be 0 */
};

struct OutputFormat {
    const char* title[2];       /* title line, may be null */
    const char* format;         /* text, with `$x' placeholders */
    const char* footer[2];      /* footer, may be null */
    const char* txt_name;       /* .txt file name */
};

struct FileFormat {
    short       block_size;               /* size of one record */
    short       entry_count;              /* # entries in file */
    short       garbage_count;            /* # garbage bytes at end */
    short       field_count;              /* # of fields in record */
    const struct Assignment* definitions; /* [field_count] */
    const struct OutputFormat* output;    /* array of output formats (0=v1 txt, 1=table, 2=v2 txt) */
    const char* dat_name;                 /* name of .dat file */
    const char* sec_name;                 /* name of %section */
    uchar*      data;                     /* data, when loaded */
    const char* item_name;                /* name of an item, for Penguin v2 record selection */
    char        checksummed;              /* nonzero if part of checksum */
};

#if 0
/**** Hull Functions ****/

struct HullFunction {
    int    hull;
    int    function;                      /* function number */
    char   is_ship;                       /* true if assignment to ship (not hull) (PHost 4) */
    char   is_player;                     /* true if assignment to player (not race) */
    char   is_default;                    /* true if this is a default assignment */
    int    level_mask;                    /* level mask */
};

extern int hullfunc_max;

void hullfunc_get_name(int number, char* out);
int  hullfunc_parse(const char* name);
void hullfunc_init(void);
void hullfunc_read(FILE* fp);
void hullfunc_write(FILE* fp);
void hullfunc_enum_for_hull(int hull, void (*func)(struct HullFunction*, void*), void* data);
#endif

/**** Utilities ****/

char* alloc_gets(FILE* fp);

#endif
